package com.dayou.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.configuration.BaseEntity;
import com.dayou.dao.DocumentMapper;
import com.dayou.dao.QuestionMapper;
import com.dayou.entity.*;
import com.dayou.service.*;
import com.dayou.vo.DocumentCycleVO;
import com.dayou.vo.DocumentVO;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/13
 * created with IntelliJ IDEA.
 */
@Service
public class DocumentServiceImpl extends ServiceImpl<DocumentMapper, Document> implements IDocumentService {

    @Autowired
    private DocumentMapper documentMapper;

    @Autowired
    private QuestionMapper questionMapper;

    @Autowired
    private IQuestionService questionService;

    @Autowired
    private IConfigurationService configurationService;

    @Autowired
    private IFailQuestionService failQuestionService;

    @Autowired
    private ICycleService cycleService;

    @Override
    public DocumentCycleVO getDocumentCycleVO() {
        Configuration mini = configurationService.getOne(new LambdaQueryWrapper<Configuration>().eq(Configuration::getName, "mini").eq(BaseEntity::getDeleted, Boolean.FALSE));
        if ("false".equals(mini.getValue())){
            return new DocumentCycleVO();
        }
        DocumentCycleVO documentCycleVO = documentMapper.getDocumentCycleVO();

        List<FailQuestion> failQuestions = failQuestionService.list(new LambdaQueryWrapper<FailQuestion>()
                .eq(FailQuestion::getCycleId, documentCycleVO.getId()).eq(BaseEntity::getDeleted, Boolean.FALSE));
        List<List<Question>> results = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(failQuestions)){
            //未通过评分项下发逻辑
            List<Long> parentIds = failQuestions.stream().map(FailQuestion::getQuestionParentId).collect(Collectors.toList());
            List<Question> fQuestions = questionService.getFailQuestions(parentIds);
            Map<Long, List<Question>> fails = fQuestions.stream().collect(Collectors.groupingBy(Question::getParentId));
            for(Map.Entry<Long,List<Question>> map :fails.entrySet()){
                results.add(map.getValue());
            }
            documentCycleVO.setQuestions(results);
        }else {
            //全量评分项下发逻辑
            Long typeId = documentCycleVO.getTypeId();
            List<Question> children = Lists.newArrayList();
            children = findChildren(typeId, children);
            Map<Long,List<Question>> result = new LinkedHashMap<>();
            Map<Long, List<Question>> collect = children.stream().collect(Collectors.groupingBy(Question::getParentId));
            collect.entrySet().stream().sorted(Map.Entry.comparingByKey()).forEachOrdered(e->result.put(e.getKey(),e.getValue()));
            for(Map.Entry<Long,List<Question>> map :result.entrySet()){
                results.add(map.getValue());
            }
            documentCycleVO.setQuestions(results);
        }
        return documentCycleVO;
    }

    @Override
    public IPage<DocumentVO> getPage(Page page, Document document) {
        IPage<DocumentVO> result = documentMapper.getPage(page,document);
        return result;
    }

    @Override
    public Cycle nextCycle(Long documentId) {
        Cycle cycle = cycleService.nextCycle(documentId);
        return cycle;
    }

    private List<Question> findChildren(Long rootId,List<Question> children){
        List<Question> list = questionMapper.findChildren(rootId);
        if (CollectionUtils.isNotEmpty(list)){
            children.addAll(list);
        }else {
            return children;
        }
        for (Question question : list){
            findChildren(question.getId(),children);
        }
        return children;
    }
}
