package com.dayou.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.configuration.BaseEntity;
import com.dayou.dao.ProfessorResultMapper;
import com.dayou.entity.Cycle;
import com.dayou.entity.ProfessorResult;
import com.dayou.entity.Question;
import com.dayou.entity.QuestionAnalysis;
import com.dayou.service.IProfessorResultService;
import com.dayou.service.IQuestionAnalysisService;
import com.dayou.service.websocket.WebSocketServer;
import com.dayou.vo.DocumentCycleVO;
import com.dayou.vo.OriginScoreVO;
import com.dayou.vo.ProfessorResultVO;
import com.dayou.vo.QuestionVO;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/13
 * created with IntelliJ IDEA.
 */
@Service
public class ProfessorResultServiceImpl extends ServiceImpl<ProfessorResultMapper, ProfessorResult> implements IProfessorResultService {

    @Autowired
    private ProfessorResultMapper professorResultMapper;

    @Autowired
    private IQuestionAnalysisService questionAnalysisService;

    @Autowired
    private WebSocketServer webSocketServer;

    @Transactional
    @Override
    public synchronized Boolean commitDocument(DocumentCycleVO documentCycleVO) {
        List<Question> questions = documentCycleVO.getResult();
        String professorNo = documentCycleVO.getProfessorNo();
        Long cycleId = documentCycleVO.getId();
        List<Long> questionIds = questions.stream().map(Question::getId).collect(Collectors.toList());
        //存在已提交的评分项再次提交
        List<ProfessorResult> list = this.list(new LambdaQueryWrapper<ProfessorResult>()
                .eq(ProfessorResult::getCycleId, cycleId).eq(ProfessorResult::getProfessorNo, professorNo).in(ProfessorResult::getQuestionId,questionIds)
                .eq(BaseEntity::getDeleted, Boolean.FALSE));
        if (CollectionUtils.isEmpty(list)){
            List<ProfessorResult> collect = questions.stream().map(x -> {
                ProfessorResult professorResult = new ProfessorResult();
                professorResult.setProfessorNo(professorNo);
                professorResult.setScore(x.getScore());
                professorResult.setCycleId(cycleId);
                professorResult.setQuestionId(x.getId());
                return professorResult;
            }).collect(Collectors.toList());
            this.saveBatch(collect);
        }

        Set<Long> allQuestionId = this.list(new LambdaQueryWrapper<ProfessorResult>().eq(ProfessorResult::getCycleId, cycleId)).stream().map(ProfessorResult::getQuestionId).collect(Collectors.toSet());
        analysisScore(cycleId,allQuestionId);
        webSocketServer.sendSocket("data");
        return Boolean.TRUE;
    }

    @Override
    public List<OriginScoreVO> getResultList(Cycle cycle) {
        List<ProfessorResultVO> list = professorResultMapper.getResultList(cycle);
        List<OriginScoreVO> result = Lists.newArrayList();
        Map<String, List<ProfessorResultVO>> professorNoMap = list.stream().collect(Collectors.groupingBy(ProfessorResultVO::getProfessorNo));
        for (Map.Entry<String,List<ProfessorResultVO>> map : professorNoMap.entrySet()){
            OriginScoreVO originScoreVO = new OriginScoreVO();
            originScoreVO.setProfessorNo(map.getKey());
            Map<String,BigDecimal> itemMap = new LinkedHashMap<>();
           // List<ProfessorResultVO> collect = map.getValue().stream().sorted((p1, p2) -> p2.getQuestionId().compareTo(p1.getQuestionId())).collect(Collectors.toList());
            map.getValue().stream().forEach(x->{
                itemMap.put(x.getQuestionName(), x.getScore());
            });
            originScoreVO.setScoreMap(itemMap);
            result.add(originScoreVO);
        }
        return result;
    }

    @Override
    public List<ProfessorResultVO> getResultByCycleId(Long id) {
        List<ProfessorResultVO> list = professorResultMapper.getResultByCycleId(id);
        return list;
    }

    @Override
    public boolean removeScore(Long id) {
        ProfessorResult pr = this.getById(id);
        this.removeById(id);
        Long cycleId = pr.getCycleId();
        Long questionId = pr.getQuestionId();
        List<QuestionAnalysis> avgScores = professorResultMapper.getAvgScore(cycleId,questionId);
        return questionAnalysisService.saveOrUpdateBatch(avgScores);
    }

    @Override
    public List<ProfessorResult> getCurrentOriginScore() {
        List<ProfessorResult> list = professorResultMapper.getCurrentOriginScore();
        return list;
    }

    @Override
    public Map<Long, BigDecimal> getCurrentAvgScore() {
        List<QuestionAnalysis> currentAvgScoreList = professorResultMapper.getCurrentAvgScore();
        Map<Long, BigDecimal> currentAvgScoreMap = currentAvgScoreList.stream().collect(Collectors.toMap(QuestionAnalysis::getQuestionId, QuestionAnalysis::getAvgScore));
        return currentAvgScoreMap;
    }

    private Boolean analysisScore(Long cycleId,Set<Long> questionIds){
        for (Long questionId : questionIds){
            List<QuestionAnalysis> avgScores = professorResultMapper.getAvgScore(cycleId,questionId);
            questionAnalysisService.saveOrUpdateBatch(avgScores);
        }
        return Boolean.TRUE;
    }
}
