package com.dayou.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.configuration.BaseEntity;
import com.dayou.dao.QuestionMapper;
import com.dayou.entity.Question;
import com.dayou.service.IQuestionService;
import com.dayou.utils.ConvertUtil;
import com.dayou.utils.TreeUtil;
import com.dayou.vo.QuestionTreeVO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/20
 * created with IntelliJ IDEA.
 */
@Service
public class QuestionServiceImpl extends ServiceImpl<QuestionMapper, Question> implements IQuestionService {
    @Autowired
    private QuestionMapper  questionMapper;
    @Override
    public List<QuestionTreeVO> getQuestionTree(Question question) {
        List<QuestionTreeVO> results = null;
        List<Question> questions = this.list(new LambdaQueryWrapper<Question>()
                .eq(question.getId()!=null,BaseEntity::getId,question.getId())
                .like(StrUtil.isNotBlank(question.getLabel()), Question::getLabel, question.getLabel())
                .eq(StrUtil.isNotBlank(question.getScope()), Question::getScope, question.getScope()));
        Map<Long, String> idLabels = questions.stream().collect(Collectors.toMap(BaseEntity::getId, Question::getLabel));
        if (CollectionUtils.isNotEmpty(questions)){
            List<QuestionTreeVO> result = ConvertUtil.copyList(questions, q -> {
                QuestionTreeVO questionTreeVO = new QuestionTreeVO();
                questionTreeVO.setId(q.getId());
                questionTreeVO.setLabel(q.getLabel());
                questionTreeVO.setParentId(q.getParentId());
                questionTreeVO.setParentLabel(idLabels.get(q.getParentId()));
                questionTreeVO.setMinScore(q.getMinScore());
                questionTreeVO.setMaxScore(q.getMaxScore());
                questionTreeVO.setScope(q.getScope());
                return questionTreeVO;
            });
            results = (List<QuestionTreeVO>) TreeUtil.buildTree(result,null);
        }
        return results;
    }

    @Override
    public List<Question> getFailQuestions(List<Long> parentIds) {
        List<Question> list = questionMapper.getFailQuestions(parentIds);
        return list;
    }
}
