package com.dayou.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.configuration.BaseEntity;
import com.dayou.configuration.ErrorCode;
import com.dayou.dao.UserMapper;
import com.dayou.dto.LoginDTO;
import com.dayou.dto.UserDTO;
import com.dayou.entity.User;
import com.dayou.service.IUserService;
import com.dayou.utils.JwtTokenUtil;
import com.dayou.utils.LoginContext;
import com.google.common.cache.Cache;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.HashMap;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/7/12
 * created with IntelliJ IDEA.
 */
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements IUserService {

    @Autowired
    @Qualifier("loginCache")
    private Cache<Long,User> loginCache;

    @Override
    public LoginDTO login(UserDTO userDTO) {
        String account = userDTO.getAccount();
        String password = userDTO.getPassword();

        User user = checkUser(account,password);

        HashMap<String, Object> claims = Maps.newHashMap();

        String token = JwtTokenUtil.doGenerateToken(claims, String.valueOf(user.getId()));

        cacheUserLoginInfo(user);
        return LoginDTO.builder().token(token).build();
    }

    private void cacheUserLoginInfo(User user) {
        loginCache.put(user.getId(),user);
        LoginContext.setUser(user);
    }

    private User checkUser(String account, String password) {
        User user = this.getOne(new LambdaQueryWrapper<User>()
                .eq(User::getAccount, account)
                .eq(User::getPassword, password)
                .eq(BaseEntity::getDeleted,Boolean.FALSE));

        if (user == null){
            ErrorCode.throwBusinessException(ErrorCode.PWD_ERROR);
        }
        return user;
    }
}
